/*
 * pnglibslb.h - internal header file.
 * Used only when compiling the shared library code,
 * to redirect standard library functions to the
 * library that was linked to the application
 *
 * Copyright (C) 2018 Thorsten Otto
 *
 * This code is released under the libpng license.
 * For conditions of distribution and use, see the disclaimer
 * and license in png.h
 */

#ifndef __PNGLIBSLB_H__
#define __PNGLIBSLB_H__ 1

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <math.h>
#include <setjmp.h>

#undef errno
#undef stderr

#undef memset
#undef memcpy
#undef memcmp

#undef strlen
#undef strcpy
#undef strcmp

#undef malloc
#undef free

#undef strerror
#undef abort

#undef remove

#undef fopen
#undef fclose
#undef fread
#undef fwrite
#undef fflush
#undef ferror

#undef gmtime

#undef pow
#undef floor
#undef frexp
#undef modf
#undef atof

#undef setjmp
#undef sigsetjmp
#undef longjmp

#include "libstrct.h"

struct _pnglibslb_funcs *pnglib_get_slb_funcs(void);

#define memset(d, c, l) pnglib_get_slb_funcs()->p_memset(d, c, l)
#define memcpy(d, s, l) pnglib_get_slb_funcs()->p_memcpy(d, s, l)
#define memcmp(d, s, l) pnglib_get_slb_funcs()->p_memcmp(d, s, l)

#define strlen pnglib_get_slb_funcs()->p_strlen
#define strcpy pnglib_get_slb_funcs()->p_strcpy
#define strcmp pnglib_get_slb_funcs()->p_strcmp

#define malloc pnglib_get_slb_funcs()->p_malloc
#define free pnglib_get_slb_funcs()->p_free

#define errno (pnglib_get_slb_funcs()->p_get_errno())
#define stderr (pnglib_get_slb_funcs()->stderr_location)
#define strerror pnglib_get_slb_funcs()->p_strerror
#define abort pnglib_get_slb_funcs()->p_abort

#define remove pnglib_get_slb_funcs()->p_remove

#define fopen pnglib_get_slb_funcs()->p_fopen
#define fclose pnglib_get_slb_funcs()->p_fclose
#define fprintf pnglib_get_slb_funcs()->p_fprintf
#define fread pnglib_get_slb_funcs()->p_fread
#define fwrite pnglib_get_slb_funcs()->p_fwrite
#define fflush pnglib_get_slb_funcs()->p_fflush
#define ferror pnglib_get_slb_funcs()->p_ferror

#define gmtime(t) pnglib_get_slb_funcs()->p_gmtime(t)

#define pow(x, y) pnglib_get_slb_funcs()->p_pow(x, y)
#define floor(x) pnglib_get_slb_funcs()->p_floor(x)
#define frexp(x, y) pnglib_get_slb_funcs()->p_frexp(x, y)
#define modf(x, y) pnglib_get_slb_funcs()->p_modf(x, y)
#define atof(x) pnglib_get_slb_funcs()->p_atof(x)

#define setjmp(j) pnglib_get_slb_funcs()->p_sigsetjmp(j, 1)
#define sigsetjmp(p, i) pnglib_get_slb_funcs()->p_sigsetjmp(p, i)
#define longjmp pnglib_get_slb_funcs()->p_longjmp

#endif /* __PNGLIBSLB_H__ */
